""" Sources from MSCI.
"""
from . import v1, v2021
from ..generic import InstrumentExporterRegistry, register_instrument_history_values_exporter

__all__ = ['v1', 'v2021']

register_instrument_history_values_exporter(
    InstrumentExporterRegistry(
        name=v1.exporters.MsciIndexExporterFactory.name,
        provider_site=v1.exporters.MsciIndexExporterFactory.provider_site,
        api_url=v1.exporters.MsciIndexExporterFactory.api_url,
        factory=v1.exporters.MsciIndexExporterFactory()
    ))

register_instrument_history_values_exporter(
    InstrumentExporterRegistry(
        name=v2021.exporters.MsciIndexExporterFactory.name,
        provider_site=v2021.exporters.MsciIndexExporterFactory.provider_site,
        api_url=v2021.exporters.MsciIndexExporterFactory.api_url,
        factory=v2021.exporters.MsciIndexExporterFactory()
    ))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Generic functionality for sources.
"""

import datetime
import inspect
import typing
import logging

from ..communication.downloader import DownloadError
from .base import (
    InstrumentStringDataDownloader, SourceDownloadError,
    InstrumentValuesHistoryParser, InstrumentInfoParser, InstrumentHistoryValuesExporter,
    InstrumentInfoProvider, InstrumentValueProvider,
    InstrumentExporterRegistry, InstrumentExporterFactory, InstrumentsInfoExporter,
    InstrumentHistoryDownloadParameters, InstrumentValuesHistoryEmpty, MaxPagesLimitExceeded)

logging.getLogger().addHandler(logging.NullHandler())


class GenericInstrumentHistoryValuesExporter(InstrumentHistoryValuesExporter):
    """ Generic, used by default, instrument history exporter.
    """
    max_paged_parameters = 10000  # limit of paged parameters

    def __init__(
            self,
            string_data_downloader: InstrumentStringDataDownloader,
            history_values_parser: InstrumentValuesHistoryParser):
        """ Initialize exporter.

        :param string_data_downloader: Used instrument string data downloader.
        :param history_values_parser: Used instrument values history parser.
        """
        self.logger = logging.getLogger(__name__ + '.' + self.__class__.__name__)

        self.string_data_downloader = string_data_downloader
        self.history_values_parser = history_values_parser

    def export_instrument_history_values(
            self,
            parameters: InstrumentHistoryDownloadParameters,
            moment_from: datetime.datetime,
            moment_to: datetime.datetime) -> typing.Iterable[InstrumentValueProvider]:

        if moment_from > moment_to:
            raise ValueError(f"Moment from ({moment_from}) is greater then moment to ({moment_to})")

        self.logger.info(f"Begin to export instrument history values "
                         f"in [{moment_from.isoformat(), moment_to.isoformat()}] "
                         f"by parameters: {parameters}")

        parameters, moment_from, moment_to = self.string_data_downloader.adjust_download_instrument_history_parameters(
            parameters=parameters,
            moment_from=moment_from,
            moment_to=moment_to)
        self.logger.info(f"Parameters was adjusted to: {parameters}")
        self.logger.info(f"Interval was adjusted to: {moment_from.isoformat()}..{moment_to.isoformat()}")

        paged_parameters_index = 0
        for paged_parameters, paged_moment_from, paged_moment_to in \
                self.string_data_downloader.paginate_download_instrument_history_parameters(
                    parameters=parameters,
                    moment_from=moment_from,
                    moment_to=moment_to):
            self.logger.info(f"Begin to export instrument history values "
                             f"by paged parameters: {paged_parameters}, "
                             f"paged interval: {paged_moment_from}..{paged_moment_to}")

            paged_parameters_index += 1
            if paged_parameters_index >= self.max_paged_parameters:
                raise MaxPagesLimitExceeded(self.max_paged_parameters)

            try:
                history_data_string_result = \
                    self.string_data_downloader.download_instrument_history_string(
                        parameters=paged_parameters,
                        moment_from=paged_moment_from,
                        moment_to=paged_moment_to)

            except DownloadError as ex:
                raise SourceDownloadError(f"Download error {ex} for parameters '{parameters}', "
                                          f"moment from '{moment_from.isoformat()}', "
                                          f"moment to '{moment_to.isoformat()}'") from ex

            try:
                values_providers = self.history_values_parser.parse(
                    history_data_string_result.downloaded_string,
                    moment_from.tzinfo)

                all_values = ((value_provider.get_instrument_value(moment_from.tzinfo), value_provider)
                              for value_provider
                              in values_providers)
                value_providers = (value_provider
                                   for value, value_provider
                                   in all_values
                                   if moment_from <= value.moment <= moment_to)

                yield from value_providers

            except InstrumentValuesHistoryEmpty:
                # history data exhausted
                history_data_string_result.set_correctness(True)
                return

            except Exception:
                history_data_string_result.set_correctness(False)
                raise

            history_data_string_result.set_correctness(True)


class GenericInstrumentsInfoExporter(InstrumentsInfoExporter):
    """ Generic, used by default, instrument info exporter.
    """

    def __init__(
            self,
            string_data_downloader: InstrumentStringDataDownloader,
            info_parser: InstrumentInfoParser):
        """ Initialize exporter.

        :param string_data_downloader: Used instrument string data downloader.
        :param info_parser: Used instrument info parser.
        """
        self.logger = logging.getLogger(__name__ + '.' + self.__class__.__name__)

        self.string_data_downloader = string_data_downloader
        self.info_parser = info_parser

    def export_instruments_info(self, parameters) -> typing.Iterator[InstrumentInfoProvider]:
        self.logger.info(f"Begin to export instruments info "
                         f"by parameters: {parameters}")

        info_data_string_result = \
            self.string_data_downloader.download_instruments_info_string(parameters=parameters)

        try:
            info_providers = self.info_parser.parse(info_data_string_result.downloaded_string)

            yield from info_providers

        except Exception:
            info_data_string_result.set_correctness(False)
            raise

        info_data_string_result.set_correctness(True)


# Global registry for all available sources and theirs factories
_GlobalInstrumentExporterRegistry: \
    typing.Dict[typing.Type[InstrumentExporterFactory], InstrumentExporterRegistry] = {}


def register_instrument_history_values_exporter(registry: InstrumentExporterRegistry):
    """ Register instrument data exporter. Validate `registry` and store it in internal global cache.

    :param registry: Instrument exporter to register.
    """
    if registry is None:
        raise ValueError("'registry' is None")
    if not isinstance(registry, InstrumentExporterRegistry):
        raise TypeError(f"'registry' is not {InstrumentExporterRegistry.__name__}: {registry!r}")

    if registry.factory is None:
        raise ValueError("'registry.factory' is None")
    if not isinstance(registry.factory, InstrumentExporterFactory):
        raise TypeError(f"'registry.factory' is not {InstrumentExporterFactory.__name__}: {registry.factory!r}")

    if registry.factory.__class__ in _GlobalInstrumentExporterRegistry:
        raise ValueError(f"Factory {registry.factory!r} already registered")

    if registry.name is None:
        raise ValueError("'registry.name' is None")
    if not str(registry.name).strip():
        raise ValueError("'registry.name' is empty or whitespace only")

    _GlobalInstrumentExporterRegistry[registry.factory.__class__] = registry


def get_all_instrument_exporters() -> typing.Tuple[InstrumentExporterRegistry, ...]:
    """ Get all available (registered) instrument data exporters.

    :return: Tuple of all available (registered) instrument data exporters
    """
    return tuple(_GlobalInstrumentExporterRegistry.values())


def get_instrument_exporter_by_factory(
        factory: typing.Union[typing.Type[InstrumentExporterFactory], InstrumentExporterFactory]) \
        -> typing.Optional[InstrumentExporterRegistry]:
    """ Find registered instrument data exporter by its factory (instance or class) and return it.

    :param factory: Factory instance or class of registered data exporter.
    :return: ``None`` if not found.
    """
    if inspect.isclass(factory):
        return _GlobalInstrumentExporterRegistry.get(factory, None)

    factory: InstrumentExporterFactory
    return _GlobalInstrumentExporterRegistry.get(factory.__class__, None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest

from sane_finances.communication.downloader import DownloadStringResult


class TestDownloadStringResult(unittest.TestCase):

    def test_Success(self):
        expected_string = 'OK'
        result = DownloadStringResult(expected_string)

        self.assertEqual(result.downloaded_string, expected_string)

        result.set_correctness(True)

        self.assertEqual(result.downloaded_string, expected_string)
        self.assertTrue(result.is_correct)

        result.set_correctness(False)

        self.assertEqual(result.downloaded_string, expected_string)
        self.assertFalse(result.is_correct)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import decimal
import re
import typing
import unittest
import datetime

from sane_finances.sources.base import ParseError, SourceDownloadError, InstrumentValuesHistoryEmpty
from sane_finances.sources.msci.v2021.meta import (
    Market, Currency, IndexLevel, Frequency, IndexSuite, Size, Style, IndexSuiteGroup, Scopes,
    IndexValue, IndexInfo, IndexPanelData)
from sane_finances.sources.msci.v2021.parsers import (
    MsciHistoryJsonParser, MsciIndexInfoParser, MsciIndexPanelDataJsonParser)

from .fakes import FakeMsciIndexDownloadParameterValuesStorage


class TestMsciHistoryJsonParser(unittest.TestCase):

    def setUp(self):
        # noinspection PyTypeChecker
        self.index_panel_data = IndexPanelData(
            markets=(Market(identity='ID', name='NAME'),),
            currencies=(Currency(identity='ID', name='NAME'),),
            index_levels=(IndexLevel(identity='ID', name='NAME'),),
            frequencies=(),
            index_suite_groups=(),
            index_suites=(),
            sizes=(),
            styles=(),
            daily_frequency=None,
            monthly_frequency=None
        )

        self.parser = MsciHistoryJsonParser(FakeMsciIndexDownloadParameterValuesStorage(self.index_panel_data))

        self.expected_level_eod_str = '12345.6789'
        self.expected_result = [IndexValue(
            calc_date=datetime.date(2000, 12, 31),
            level_eod=decimal.Decimal(self.expected_level_eod_str),
            msci_index_code='990300',
            index_variant_type=self.index_panel_data.index_levels[0],
            currency=self.index_panel_data.currencies[0])]

        self.expected_date_str = self.expected_result[0].calc_date.strftime(self.parser.date_format)

    def check_parse_raise(self, error: typing.Type[Exception], invalid_json: str):
        with self.assertRaises(error):
            _ = list(self.parser.parse(invalid_json, tzinfo=None))

    def test_parse_Success(self):
        valid_json = f"""
{{
    "msci_index_code":"{self.expected_result[0].msci_index_code}",
    "index_variant_type":"{self.expected_result[0].index_variant_type.identity}",
    "ISO_currency_symbol":"{self.expected_result[0].currency.identity}",
    "indexes":{{
        "INDEX_LEVELS":[{{"level_eod":{self.expected_level_eod_str},"calc_date":{self.expected_date_str}}}]
    }}
}}
"""
        result = list(self.parser.parse(valid_json, tzinfo=None))

        self.assertSequenceEqual(result, self.expected_result)

    def test_parse_raisesWhenUnknownLevel(self):
        invalid_json = f"""
        {{
            "msci_index_code":"{self.expected_result[0].msci_index_code}",
            "index_variant_type":"WRONG",
            "ISO_currency_symbol":"{self.expected_result[0].currency.identity}",
            "indexes":{{
                "INDEX_LEVELS":[{{"level_eod":{self.expected_level_eod_str},"calc_date":{self.expected_date_str}}}]
            }}
        }}
        """
        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenUnknownCurrency(self):
        invalid_json = f"""
        {{
            "msci_index_code":"{self.expected_result[0].msci_index_code}",
            "index_variant_type":"{self.expected_result[0].index_variant_type.identity}",
            "ISO_currency_symbol":"WRONG",
            "indexes":{{
                "INDEX_LEVELS":[{{"level_eod":{self.expected_level_eod_str},"calc_date":{self.expected_date_str}}}]
            }}
        }}
        """
        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenEmptyHistory(self):
        valid_json = f"""
        {{
            "msci_index_code":"{self.expected_result[0].msci_index_code}",
            "index_variant_type":"{self.expected_result[0].index_variant_type.identity}",
            "ISO_currency_symbol":"{self.expected_result[0].currency.identity}",
            "indexes":{{
                "INDEX_LEVELS":[]
            }}
        }}
        """
        self.check_parse_raise(InstrumentValuesHistoryEmpty, valid_json)

    def test_parse_raisesWhenWrongValue(self):
        invalid_json = f"""
        {{
            "msci_index_code":"{self.expected_result[0].msci_index_code}",
            "index_variant_type":"{self.expected_result[0].index_variant_type.identity}",
            "ISO_currency_symbol":"{self.expected_result[0].currency.identity}",
            "indexes":{{
                "INDEX_LEVELS":[{{"level_eod":"WRONG","calc_date":{self.expected_date_str}}}]
            }}
        }}
        """
        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenWrongDate(self):
        invalid_json = f"""
        {{
            "msci_index_code":"{self.expected_result[0].msci_index_code}",
            "index_variant_type":"{self.expected_result[0].index_variant_type.identity}",
            "ISO_currency_symbol":"{self.expected_result[0].currency.identity}",
            "indexes":{{
                "INDEX_LEVELS":[{{"level_eod":{self.expected_level_eod_str},"calc_date":"WRONG"}}]
            }}
        }}
        """
        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenNoCode(self):
        invalid_json = f"""
        {{
            "msci_index_code__":"{self.expected_result[0].msci_index_code}",
            "index_variant_type":"{self.expected_result[0].index_variant_type.identity}",
            "ISO_currency_symbol":"{self.expected_result[0].currency.identity}",
            "indexes":{{
                "INDEX_LEVELS":[{{"level_eod":{self.expected_level_eod_str},"calc_date":{self.expected_date_str}}}]
            }}
        }}
        """
        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenNoCurrency(self):
        invalid_json = f"""
        {{
            "msci_index_code":"{self.expected_result[0].msci_index_code}",
            "index_variant_type":"{self.expected_result[0].index_variant_type.identity}",
            "ISO_currency_symbol__":"{self.expected_result[0].currency.identity}",
            "indexes":{{
                "INDEX_LEVELS":[{{"level_eod":{self.expected_level_eod_str},"calc_date":{self.expected_date_str}}}]
            }}
        }}
        """
        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenNoVariantType(self):
        invalid_json = f"""
        {{
            "msci_index_code":"{self.expected_result[0].msci_index_code}",
            "index_variant_type__":"{self.expected_result[0].index_variant_type.identity}",
            "ISO_currency_symbol":"{self.expected_result[0].currency.identity}",
            "indexes":{{
                "INDEX_LEVELS":[{{"level_eod":{self.expected_level_eod_str},"calc_date":{self.expected_date_str}}}]
            }}
        }}
        """
        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenNoIndexes(self):
        invalid_json = f"""
        {{
            "msci_index_code":"{self.expected_result[0].msci_index_code}",
            "index_variant_type":"{self.expected_result[0].index_variant_type.identity}",
            "ISO_currency_symbol":"{self.expected_result[0].currency.identity}",
            "indexes__":{{
                "INDEX_LEVELS":[{{"level_eod":{self.expected_level_eod_str},"calc_date":{self.expected_date_str}}}]
            }}
        }}
        """
        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenWrongIndexes(self):
        invalid_json = f"""
        {{
            "msci_index_code":"{self.expected_result[0].msci_index_code}",
            "index_variant_type":"{self.expected_result[0].index_variant_type.identity}",
            "ISO_currency_symbol":"{self.expected_result[0].currency.identity}",
            "indexes":[]
        }}
        """
        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenNoIndexLevels(self):
        invalid_json = f"""
        {{
            "msci_index_code":"{self.expected_result[0].msci_index_code}",
            "index_variant_type":"{self.expected_result[0].index_variant_type.identity}",
            "ISO_currency_symbol":"{self.expected_result[0].currency.identity}",
            "indexes":{{
                "INDEX_LEVELS__":[{{"level_eod":{self.expected_level_eod_str},"calc_date":{self.expected_date_str}}}]
            }}
        }}
        """
        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenWrongIndexLevels(self):
        invalid_json = f"""
        {{
            "msci_index_code":"{self.expected_result[0].msci_index_code}",
            "index_variant_type":"{self.expected_result[0].index_variant_type.identity}",
            "ISO_currency_symbol":"{self.expected_result[0].currency.identity}",
            "indexes":{{
                "INDEX_LEVELS":[42]
            }}
        }}
        """
        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenNoLevel(self):
        invalid_json = f"""
        {{
            "msci_index_code":"{self.expected_result[0].msci_index_code}",
            "index_variant_type":"{self.expected_result[0].index_variant_type.identity}",
            "ISO_currency_symbol":"{self.expected_result[0].currency.identity}",
            "indexes":{{
                "INDEX_LEVELS":[{{"level_eod__":{self.expected_level_eod_str},"calc_date":{self.expected_date_str}}}]
            }}
        }}
        """
        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenNoDate(self):
        invalid_json = f"""
        {{
            "msci_index_code":"{self.expected_result[0].msci_index_code}",
            "index_variant_type":"{self.expected_result[0].index_variant_type.identity}",
            "ISO_currency_symbol":"{self.expected_result[0].currency.identity}",
            "indexes":{{
                "INDEX_LEVELS":[{{"level_eod":{self.expected_level_eod_str},"calc_date__":{self.expected_date_str}}}]
            }}
        }}
        """
        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenNoData(self):
        invalid_json = ''

        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenSourceError(self):
        invalid_json = """
{
    "timestamp":"Aug 14, 2021 4:20:12 PM",
    "user":"sys_x_bmapip01",
    "error_code":" 100",
    "error_message":" null Invalid Parameter end_date : '19691231. Calculation date cannot be earlier than 19970101'"
}
"""
        self.check_parse_raise(SourceDownloadError, invalid_json)

    def test_parse_raisesWhenWrongJson(self):
        invalid_json = """[]"""

        self.check_parse_raise(ParseError, invalid_json)

    def test_parse_raisesWhenUnknownJson(self):
        invalid_json = """{}"""

        self.check_parse_raise(ParseError, invalid_json)


class TestMsciIndexInfoParser(unittest.TestCase):

    def setUp(self):
        self.parser = MsciIndexInfoParser()

    def test_parse_Success(self):
        expected_result = [IndexInfo(msci_index_code='INDEX CODE', index_name='INDEX NAME')]

        json = f"""{{"indexes":[
        {{"msci_index_code":"{expected_result[0].msci_index_code}","index_name":"{expected_result[0].index_name}"}}
        ]}}"""

        result = list(self.parser.parse(json))

        self.assertSequenceEqual(result, expected_result)

    def test_parse_SuccessEmptyList(self):
        expected_result = []

        json = """{"indexes":[]}"""

        result = list(self.parser.parse(json))

        self.assertSequenceEqual(result, expected_result)

    def test_parse_raisesEmptyString(self):
        wrong_json = ''

        with self.assertRaises(ParseError):
            list(self.parser.parse(wrong_json))

    def test_parse_raisesWrongJson(self):
        wrong_json = '[]'

        with self.assertRaises(ParseError):
            list(self.parser.parse(wrong_json))

    def test_parse_raisesNoIndexes(self):
        wrong_json = """{}"""

        with self.assertRaises(ParseError):
            list(self.parser.parse(wrong_json))

    def test_parse_raisesWrongIndexes(self):
        wrong_json = """{"indexes":[1]}"""

        with self.assertRaises(ParseError):
            list(self.parser.parse(wrong_json))

    def test_parse_raisesNoCode(self):
        wrong_json = """{"indexes":[{"msci_index_code__":"CODE","index_name":"NAME"}]}"""

        with self.assertRaises(ParseError):
            list(self.parser.parse(wrong_json))

    def test_parse_raisesNoName(self):
        wrong_json = """{"indexes":[{"msci_index_code":"CODE","index_name__":"NAME"}]}"""

        with self.assertRaises(ParseError):
            list(self.parser.parse(wrong_json))


class TestMsciIndexPanelDataJsonParser(unittest.TestCase):

    def setUp(self):
        self.daily_frequency = Frequency(identity='DAILY', name='Daily')
        self.monthly_frequency = Frequency(identity='END_OF_MONTH', name='Monthly')
        index_suite_group = IndexSuiteGroup(name='Capped')
        self.expected_result = IndexPanelData(
            markets=(Market(identity='24576', name='All Country (DM+EM)', scope=Scopes.REGIONAL),
                     Market(identity='16384', name='Developed Markets (DM)'),),
            currencies=(Currency(identity='USD', name='USD'),),
            index_levels=(IndexLevel(identity='STRD', name='Price'),),
            frequencies=(self.daily_frequency, self.monthly_frequency),
            index_suite_groups=(index_suite_group,),
            index_suites=(IndexSuite(identity='C', name='None'),
                          IndexSuite(identity='C', name='None', group=index_suite_group),),
            sizes=(Size(identity='ID', name='NAME'),),
            styles=(Style(identity='ID', name='NAME'),),
            daily_frequency=self.daily_frequency,
            monthly_frequency=self.monthly_frequency
        )

        self.parser = MsciIndexPanelDataJsonParser()

    def generate_valid_json(self):
        valid_json = f"""
        {{
            "market":[{{"id": "{self.expected_result.markets[0].identity}",
                        "name": "{self.expected_result.markets[0].name}"}},
                      {{"id": "{self.expected_result.markets[1].identity}",
                        "name": "{self.expected_result.markets[1].name}"}}],
            "currency":[{{"id": "{self.expected_result.currencies[0].identity}",
                          "name": "{self.expected_result.currencies[0].name}"}}],
            "indexLevels":[{{"id":"{self.expected_result.index_levels[0].identity}",
                             "name":"{self.expected_result.index_levels[0].name}"}}],
            "frequency":[{{"id":"{self.daily_frequency.identity}",
                           "name":"{self.daily_frequency.name}"}},
                         {{"id":"{self.monthly_frequency.identity}",
                           "name":"{self.monthly_frequency.name}"}}],
            "indexSuite":[
                {{
                    "id":"{self.expected_result.index_suites[0].identity}",
                    "name":"{self.expected_result.index_suites[0].name}"
                }},
                {{ 
                    "optgroup":"{self.expected_result.index_suite_groups[0].name}",
                    "options":[{{"id":"{self.expected_result.index_suites[1].identity}",
                                 "name":"{self.expected_result.index_suites[1].name}"}}]
                }}
            ],
            "size":[{{"id":"{self.expected_result.sizes[0].identity}",
                      "name":"{self.expected_result.sizes[0].name}"}}],
            "style":[{{"id":"{self.expected_result.styles[0].identity}",
                       "name":"{self.expected_result.styles[0].name}"}}]    
        }}"""

        return valid_json

    def spoil_block(self, block_name: str, new_content: str):
        spoil_pattern = re.compile(fr'"{block_name}"\s*:\s*\[.*?]', re.IGNORECASE | re.MULTILINE | re.DOTALL)
        return spoil_pattern.sub(f'"{block_name}":[{new_content}]', self.generate_valid_json())

    def test_parse_Success(self):
        result = self.parser.parse(self.generate_valid_json())

        self.assertEqual(result, self.expected_result)

    def test_parse_raisesEmptyString(self):
        wrong_json = ''

        with self.assertRaises(ParseError):
            list(self.parser.parse(wrong_json))

    def test_parse_raisesWrongJson(self):
        wrong_json = '[]'

        with self.assertRaises(ParseError):
            list(self.parser.parse(wrong_json))

    def test_parse_raisesNotFoundDailyFrequency(self):
        self.daily_frequency = Frequency(identity='WRONG', name='WRONG')
        wrong_json = self.generate_valid_json()

        with self.assertRaises(ParseError):
            list(self.parser.parse(wrong_json))

    def test_parse_raisesNotFoundMonthlyFrequency(self):
        self.monthly_frequency = Frequency(identity='WRONG', name='WRONG')
        wrong_json = self.generate_valid_json()

        with self.assertRaises(ParseError):
            list(self.parser.parse(wrong_json))

    def test_parse_raisesWhenWrongBlockContent(self):
        for block_name, spoil_content in (('market', True),
                                           ('currency', True),
                                           ('indexLevels', True),
                                           ('frequency', True),
                                           ('indexSuite', False),
                                           ('size', True),
                                           ('style', True)):
            # no block
            wrong_json = self.generate_valid_json().replace(f'"{block_name}"', f'"{block_name}__"')

            with self.assertRaises(ParseError):
                list(self.parser.parse(wrong_json))

            if spoil_content:
                # wrong element type
                wrong_json = self.spoil_block(block_name, '0')

                with self.assertRaises(ParseError):
                    list(self.parser.parse(wrong_json))

                # no ID
                wrong_json = self.spoil_block(block_name, '{"id__":"ID","name":"NAME"}')

                with self.assertRaises(ParseError):
                    list(self.parser.parse(wrong_json))

                # no name
                wrong_json = self.spoil_block(block_name, '{"id":"ID","name__":"NAME"}')

                with self.assertRaises(ParseError):
                    list(self.parser.parse(wrong_json))
